var aliases = [
  ["CustomEvent", "CustomEvent"],
  ["Event", "Event"],
  ["Events", "Event"],
  ["HTMLEvents", "Event"],
  ["KeyboardEvent", "KeyboardEvent"],
  ["KeyEvents", "KeyboardEvent"],
  ["MessageEvent", "MessageEvent"],
  ["MouseEvent", "MouseEvent"],
  ["MouseEvents", "MouseEvent"],
  ["TouchEvent", "TouchEvent"],
  ["UIEvent", "UIEvent"],
  ["UIEvents", "UIEvent"]
];
